@@ -20,7 +20,7 @@ import os
 from pathlib import Path
 from locale import gettext as _
 
-from gi.repository import Gtk, GLib, Gio, Handy
+from gi.repository import Gtk, GLib, Gio, Handy, GObject
 
 from . import utils
 from . import logger
@@ -30,6 +30,7 @@ from .worker import PortfolioCopyWorker
 from .worker import PortfolioDeleteWorker
 from .worker import PortfolioLoadWorker
 from .worker import PortfolioOpenWorker
+from .worker import PortfolioPropertiesWorker
 from .places import PortfolioPlaces
 
 
@@ -47,8 +48,8 @@ class PortfolioWindow(Handy.ApplicationWindow):
     previous = Gtk.Template.Child()
     next = Gtk.Template.Child()
     search = Gtk.Template.Child()
-    back = Gtk.Template.Child()
     rename = Gtk.Template.Child()
+    detail = Gtk.Template.Child()
     delete = Gtk.Template.Child()
     cut = Gtk.Template.Child()
     copy = Gtk.Template.Child()
@@ -66,6 +67,8 @@ class PortfolioWindow(Handy.ApplicationWindow):
     show_hidden_button = Gtk.Template.Child()
     a_to_z_button = Gtk.Template.Child()
     go_top_button = Gtk.Template.Child()
+    about_back_button = Gtk.Template.Child()
+    properties_back_button = Gtk.Template.Child()
 
     search_box = Gtk.Template.Child()
     search_entry = Gtk.Template.Child()
@@ -77,26 +80,40 @@ class PortfolioWindow(Handy.ApplicationWindow):
     selection_tools = Gtk.Template.Child()
     navigation_tools = Gtk.Template.Child()
     places_box = Gtk.Template.Child()
+    places_inner_box = Gtk.Template.Child()
     content_stack = Gtk.Template.Child()
     loading_box = Gtk.Template.Child()
     content_box = Gtk.Template.Child()
-    app_box = Gtk.Template.Child()
+    files_box = Gtk.Template.Child()
     about_box = Gtk.Template.Child()
     close_box = Gtk.Template.Child()
     close_tools = Gtk.Template.Child()
     stop_box = Gtk.Template.Child()
     stop_tools = Gtk.Template.Child()
-    deck = Gtk.Template.Child()
+    about_deck = Gtk.Template.Child()
+    content_deck = Gtk.Template.Child()
     headerbar = Gtk.Template.Child()
     placeholder_box = Gtk.Template.Child()
     menu_box = Gtk.Template.Child()
+    menu_popover = Gtk.Template.Child()
+    menu_button = Gtk.Template.Child()
+    home_menu_button = Gtk.Template.Child()
     content_scroll = Gtk.Template.Child()
     go_top_revealer = Gtk.Template.Child()
+    properties_box = Gtk.Template.Child()
+    property_name = Gtk.Template.Child()
+    property_location = Gtk.Template.Child()
+    property_type = Gtk.Template.Child()
+    property_size = Gtk.Template.Child()
+    property_created = Gtk.Template.Child()
+    property_modified = Gtk.Template.Child()
+    property_accessed = Gtk.Template.Child()
 
     ICON_COLUMN = 0
     NAME_COLUMN = 1
     PATH_COLUMN = 2
     SEARCH_DELAY = 500
+    LOAD_ANIMATION_DELAY = 250
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
@@ -111,13 +128,17 @@ class PortfolioWindow(Handy.ApplicationWindow):
         self._editing = False
         self._to_copy = []
         self._to_cut = []
+        self._to_select = None
+        self._to_select_row = None
         self._last_clicked = None
+        self._last_vscroll_value = None
         self._dont_activate = False
         self._force_select = False
         self._force_go_home = False
         self._history = []
         self._index = -1
         self._search_delay_handler_id = 0
+        self._load_delay_handler_id = 0
 
         self.gesture = Gtk.GestureLongPress.new(self.treeview)
         self.gesture.connect("pressed", self._on_long_pressed)
@@ -136,6 +157,7 @@ class PortfolioWindow(Handy.ApplicationWindow):
         self.previous.connect("clicked", self._on_go_previous)
         self.next.connect("clicked", self._on_go_next)
         self.rename.connect("clicked", self._on_rename_clicked)
+        self.detail.connect("clicked", self._on_detail_clicked)
         self.delete.connect("clicked", self._on_delete_clicked)
         self.cut.connect("clicked", self._on_cut_clicked)
         self.copy.connect("clicked", self._on_copy_clicked)
@@ -150,6 +172,12 @@ class PortfolioWindow(Handy.ApplicationWindow):
         self.a_to_z_button.connect("toggled", self._on_sort_toggled)
         self.go_top_button.connect("clicked", self._go_to_top)
         self.stop_button.connect("clicked", self._on_stop_clicked)
+        self.about_back_button.connect("clicked", self._on_about_back_clicked)
+        self.properties_back_button.connect("clicked", self._on_properties_back_clicked)
+
+        # XXX no model for options yet so this...
+        self.menu_button.connect("clicked", self._on_menu_button_clicked)
+        self.home_menu_button.connect("clicked", self._on_menu_button_clicked)
 
         self._adjustment = self.content_scroll.get_vadjustment()
         self._adjustment.connect("value-changed", self._update_go_top_button)
@@ -157,14 +185,58 @@ class PortfolioWindow(Handy.ApplicationWindow):
         self.search.connect("toggled", self._on_search_toggled)
         self.search_entry.connect("search-changed", self._on_search_changed)
         self.search_entry.connect("stop-search", self._on_search_stopped)
-        self.back.connect("clicked", self._on_back_clicked)
 
         places = PortfolioPlaces()
         places.connect("updated", self._on_places_updated)
         places.connect("removed", self._on_places_removed)
-        self.places_box.add(places)
+        self.places_inner_box.add(places)
 
-        self.open()
+        self._properties = PortfolioPropertiesWorker()
+        self._properties.bind_property(
+            "name",
+            self.property_name,
+            "label",
+            GObject.BindingFlags.SYNC_CREATE,
+        )
+        self._properties.bind_property(
+            "location",
+            self.property_location,
+            "label",
+            GObject.BindingFlags.SYNC_CREATE,
+        )
+        self._properties.bind_property(
+            "type",
+            self.property_type,
+            "label",
+            GObject.BindingFlags.SYNC_CREATE,
+        )
+        self._properties.bind_property(
+            "size",
+            self.property_size,
+            "label",
+            GObject.BindingFlags.SYNC_CREATE,
+        )
+        self._properties.bind_property(
+            "created",
+            self.property_created,
+            "label",
+            GObject.BindingFlags.SYNC_CREATE,
+        )
+        self._properties.bind_property(
+            "modified",
+            self.property_modified,
+            "label",
+            GObject.BindingFlags.SYNC_CREATE,
+        )
+        self._properties.bind_property(
+            "accessed",
+            self.property_accessed,
+            "label",
+            GObject.BindingFlags.SYNC_CREATE,
+        )
+
+        self.content_deck.connect("notify::visible-child", self._on_content_folded)
+        self.connect("destroy", self._on_shutdown)
 
     def _filter(self, model, row, data=None):
         path = model[row][self.PATH_COLUMN]
@@ -227,6 +299,27 @@ class PortfolioWindow(Handy.ApplicationWindow):
         self.selection.select_iter(row)
         self._force_select = False
 
+    def _select_and_go(self, row, edit=False):
+        result, row = self.filtered.convert_child_iter_to_iter(row)
+        result, row = self.sorted.convert_child_iter_to_iter(row)
+
+        self._select_row(row)
+        GLib.idle_add(self._go_to_selection)
+
+        if edit is True:
+            GLib.timeout_add(100, self._wait_and_edit)
+
+    def _wait_and_edit(self):
+        value = self._adjustment.get_value()
+
+        if value == self._last_vscroll_value:
+            self._on_rename_clicked(None)
+            self._last_vscroll_value = None
+            return False
+
+        self._last_vscroll_value = value
+        return True
+
     def _get_selection(self):
         model, treepaths = self.selection.get_selected_rows()
         selection = [
@@ -249,6 +342,11 @@ class PortfolioWindow(Handy.ApplicationWindow):
         self.liststore.remove(self.liststore.get_iter(treepath))
 
     def _populate(self, directory):
+        self._switch_to_navigation_mode()
+
+        if self._worker is not None:
+            self._worker.stop()
+
         self._worker = PortfolioLoadWorker(
             directory, self.show_hidden_button.props.active
         )
@@ -310,6 +408,9 @@ class PortfolioWindow(Handy.ApplicationWindow):
         if len(self.sorted) >= 1:
             self.treeview.scroll_to_cell(0, None, True, 0, 0)
 
+    def _go_back_to_homepage(self):
+        self.content_deck.set_visible_child(self.places_box)
+
     def _move(self, path, navigating=False):
         self._clean_popups()
 
@@ -333,9 +434,11 @@ class PortfolioWindow(Handy.ApplicationWindow):
         self._history = []
         self._index = -1
         self._move(path, False)
+        self._update_mode()
 
     def _refresh(self):
-        self._move(self._history[self._index], True)
+        if self._index > -1:
+            self._move(self._history[self._index], True)
 
     def _switch_to_navigation_mode(self):
         self.selection.set_mode(Gtk.SelectionMode.NONE)
@@ -371,6 +474,11 @@ class PortfolioWindow(Handy.ApplicationWindow):
     def _clean_progress(self):
         self.loading_description.set_text("")
 
+    def _clean_loading_delay(self):
+        if self._load_delay_handler_id != 0:
+            GLib.Source.remove(self._load_delay_handler_id)
+            self._load_delay_handler_id = 0
+
     def _update_mode(self):
         count = self.selection.count_selected_rows()
         if count == 0:
@@ -384,6 +492,7 @@ class PortfolioWindow(Handy.ApplicationWindow):
 
     def _update_all(self):
         self._update_search()
+        self._update_treeview()
         self._update_content_stack()
         self._update_navigation()
         self._update_navigation_tools()
@@ -399,6 +508,10 @@ class PortfolioWindow(Handy.ApplicationWindow):
         self.search.props.sensitive = sensitive
         self.search_entry.sensitive = sensitive
 
+    def _update_treeview(self):
+        sensitive = not self._busy
+        self.treeview.props.sensitive = sensitive
+
     def _update_content_stack(self):
         if self._busy:
             return
@@ -416,7 +529,7 @@ class PortfolioWindow(Handy.ApplicationWindow):
             self.next.props.sensitive = False
             return
 
-        self.previous.props.sensitive = True if self._index > 0 else False
+        self.previous.props.sensitive = True
         self.next.props.sensitive = (
             True if len(self._history) - 1 > self._index else False
         )
@@ -448,6 +561,7 @@ class PortfolioWindow(Handy.ApplicationWindow):
         self.copy.props.sensitive = sensitive
 
         self._update_rename()
+        self._update_detail()
 
     def _update_navigation_tools(self):
         count = self.selection.count_selected_rows()
@@ -461,6 +575,11 @@ class PortfolioWindow(Handy.ApplicationWindow):
         sensitive = count == 1 and not self._editing and not self._busy
         self.rename.props.sensitive = sensitive
 
+    def _update_detail(self):
+        count = self.selection.count_selected_rows()
+        sensitive = count == 1 and not self._editing and not self._busy
+        self.detail.props.sensitive = sensitive
+
     def _update_directory_title(self):
         directory = self._history[self._index]
         name = os.path.basename(directory)
@@ -516,37 +635,64 @@ class PortfolioWindow(Handy.ApplicationWindow):
     def _on_load_started(self, worker, directory):
         self._busy = True
 
-        self._update_directory_title()
-        self._reset_search()
-
         self.liststore.clear()
 
+        self._update_directory_title()
+        self._reset_search()
+        self._update_all()
+
+        self._load_delay_handler_id = GLib.timeout_add(
+            self.LOAD_ANIMATION_DELAY,
+            self._on_load_started_delayed,
+        )
+
+    def _on_load_started_delayed(self):
         self.loading_label.set_text(_("Loading"))
         self.loading_bar.set_fraction(0.0)
         self.content_stack.set_visible_child(self.loading_box)
 
         self._update_all()
 
+        self._load_delay_handler_id = 0
+        return GLib.SOURCE_REMOVE
+
     def _on_load_updated(self, worker, directory, found, index, total):
         for name, path in found:
             icon = self._find_icon(path)
-            self.liststore.append([icon, name, path])
+            row = self.liststore.append([icon, name, path])
+
+            if self._to_select == path:
+                self._to_select_row = row
+
         self.loading_bar.set_fraction((index + 1) / total)
 
     def _on_load_finished(self, worker, directory):
         self._busy = False
         self._clean_workers()
+        self._clean_loading_delay()
 
-        self._go_to_top()
         self._update_all()
 
+        if self._to_select_row is not None:
+            self._switch_to_selection_mode()
+            self._select_and_go(self._to_select_row)
+            self._to_select_row = None
+            self._to_select = None
+        else:
+            self._go_to_top()
+
     def _on_load_failed(self, worker, directory):
         self._busy = False
         self._clean_workers()
+        self._clean_loading_delay()
 
         name = os.path.basename(directory)
+        self.loading_label.set_text(_("Loading"))
         self.loading_description.set_text(_("Could not load %s") % name)
+        self.content_stack.set_visible_child(self.loading_box)
 
+        self._to_select_row = None
+        self._to_select = None
         self._force_go_home = True
         self.action_stack.set_visible_child(self.close_box)
         self.tools_stack.set_visible_child(self.close_tools)
@@ -585,8 +731,11 @@ class PortfolioWindow(Handy.ApplicationWindow):
         self._update_mode()
 
     def _on_go_previous(self, button):
-        self._index -= 1
-        self._move(self._history[self._index], True)
+        if self._index == 0:
+            self._go_back_to_homepage()
+        else:
+            self._index -= 1
+            self._move(self._history[self._index], True)
 
     def _on_go_next(self, button):
         self._index += 1
@@ -606,6 +755,12 @@ class PortfolioWindow(Handy.ApplicationWindow):
     def _on_search_stopped(self, entry):
         self._reset_search()
 
+    def _on_detail_clicked(self, button):
+        model, treepaths = self.selection.get_selected_rows()
+        treepath = treepaths[-1]
+        path = model[treepath][self.PATH_COLUMN]
+        self.show_properties(path)
+
     def _on_rename_clicked(self, button):
         self.name_cell.props.editable = True
         model, treepaths = self.selection.get_selected_rows()
@@ -680,7 +835,7 @@ class PortfolioWindow(Handy.ApplicationWindow):
         else:
             name = _("these %d files") % count
 
-        description = _("Delete %s?") % name
+        description = _("Delete %s permanently?") % name
 
         self._notify(
             description,
@@ -873,7 +1028,7 @@ class PortfolioWindow(Handy.ApplicationWindow):
             return
 
         self._force_go_home = False
-        self._reset_to_path(PortfolioPlaces.PORTFOLIO_HOME_DIR)
+        self._go_back_to_homepage()
 
     def _on_stop_clicked(self, button):
         self.loading_label.set_text(_("Stopping"))
@@ -908,13 +1063,7 @@ class PortfolioWindow(Handy.ApplicationWindow):
 
         icon = self._find_icon(path)
         row = self.liststore.append([icon, folder_name, path])
-        result, row = self.filtered.convert_child_iter_to_iter(row)
-        result, row = self.sorted.convert_child_iter_to_iter(row)
-
-        self._select_row(row)
-
-        self._go_to_selection()
-        self._on_rename_clicked(None)
+        self._select_and_go(row, edit=True)
 
     def _on_row_activated(self, treeview, treepath, treecolumn, data=None):
         if self._dont_activate is True:
@@ -926,30 +1075,45 @@ class PortfolioWindow(Handy.ApplicationWindow):
 
     def _on_places_updated(self, button, path):
         self._reset_to_path(path)
+        self.content_deck.set_visible_child(self.files_box)
 
     def _on_places_removed(self, button, path):
+        if self._index == -1:
+            return
         directory = self._history[self._index]
         if not directory.startswith(path):
             return
-        self._reset_to_path(PortfolioPlaces.PORTFOLIO_HOME_DIR)
+        self._go_back_to_homepage()
 
     def _on_help_clicked(self, button):
         Gio.AppInfo.launch_default_for_uri("https://github.com/tchx84/Portfolio", None)
 
     def _on_about_clicked(self, button):
-        self.deck.set_visible_child(self.about_box)
+        self.about_deck.set_visible_child(self.about_box)
 
-    def _on_back_clicked(self, button):
-        self.deck.set_visible_child(self.app_box)
+    def _on_about_back_clicked(self, button):
+        self.about_deck.set_visible_child(self.content_deck)
+
+    def _on_properties_back_clicked(self, button):
+        self.content_deck.set_visible_child(self.files_box)
 
     def _on_long_pressed(self, gesture, x, y):
-        if self.selection.get_mode() != Gtk.SelectionMode.MULTIPLE:
-            self._switch_to_selection_mode()
-            treepath = self.treeview.get_path_at_pos(x, y)[0]
-            self.selection.select_path(treepath)
-            # because of the custom selection rules, is not guaranteed
-            # that this will actually be selected so always update mode.
-            self._update_mode()
+        if self.selection.get_mode() == Gtk.SelectionMode.MULTIPLE:
+            return
+
+        self._switch_to_selection_mode()
+        path = self.treeview.get_path_at_pos(x, y)
+
+        if path is None:
+            self._switch_to_navigation_mode()
+            return
+
+        treepath = path[0]
+        self.selection.select_path(treepath)
+
+        # because of the custom selection rules, is not guaranteed
+        # that this will actually be selected so always update mode.
+        self._update_mode()
 
     def _on_hidden_toggled(self, button):
         self._refresh()
@@ -957,11 +1121,36 @@ class PortfolioWindow(Handy.ApplicationWindow):
     def _on_sort_toggled(self, button):
         self._refresh()
 
-    def open(self, path=PortfolioPlaces.PORTFOLIO_HOME_DIR):
-        if not os.path.isdir(path):
+    def _on_menu_button_clicked(self, button):
+        button.props.popover = self.menu_popover
+        self.menu_popover.popup()
+
+    def _on_content_folded(self, deck, data=None):
+        child = self.content_deck.get_visible_child()
+        if child == self.places_box and self._worker is not None:
+            self._worker.stop()
+        elif child == self.files_box:
+            self._properties.stop()
+
+    def _on_shutdown(self, window):
+        if self._worker is not None:
+            self._worker.stop()
+        self._properties.stop()
+
+    def open(self, path=PortfolioPlaces.PORTFOLIO_HOME_DIR, force_page_switch=False):
+        # XXX so cheap !
+        path = path.replace("file://", "")
+
+        # make sure it exists though !
+        if not os.path.exists(path):
             logger.warning(_("Could not open %s") % path)
             return
 
+        # if it's a file then use its parent folder
+        if not os.path.isdir(path):
+            self._to_select = path
+            path = os.path.dirname(path)
+
         # XXX no support for background workers yet
         if self._busy and not isinstance(self._worker, PortfolioLoadWorker):
             logger.warning(_("Could not open %s") % path)
@@ -970,5 +1159,20 @@ class PortfolioWindow(Handy.ApplicationWindow):
         if isinstance(self._worker, PortfolioLoadWorker):
             self._worker.stop()
             self._clean_workers()
+            self._clean_loading_delay()
 
         self._reset_to_path(path)
+
+        if force_page_switch is False:
+            return
+
+        self.about_deck.set_visible_child(self.content_deck)
+        self.content_deck.set_visible_child(self.files_box)
+
+    def show_properties(self, path, force_page_switch=False):
+        self._properties.props.path = path
+
+        if force_page_switch is True:
+            self.open(path, force_page_switch)
+
+        self.content_deck.set_visible_child(self.properties_box)
